<?php 

namespace Drupal\file_table_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\Annotation\FieldFormatter;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Plugin\Field\FieldFormatter\FileFormatterBase;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\file_table_formatter\FileTableFormatterProcessorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @FieldFormatter(
 *  id = "file_table_formatter",
 *  label = @Translation("Display file contents as table"),
 *  field_types = {"file"}
 * )
 */
class FileTableFormatter extends FileFormatterBase {

  use StringTranslationTrait;

  /**
   * {@inheritDoc}
   *
   * @param mixed $plugin_id
   * @param mixed $plugin_definition
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   * @param mixed[] $settings
   * @param mixed $label
   * @param mixed $view_mode
   * @param mixed[] $third_party_settings
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   * @param \Drupal\file_table_formatter\FileTableFormatterProcessorInterface $fileTableFormatterProcessor,
   */
  public function __construct(
    $plugin_id,
    $plugin_definition,
    FieldDefinitionInterface $field_definition,
    array $settings,
    $label,
    $view_mode,
    array $third_party_settings,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected FileTableFormatterProcessorInterface $fileTableFormatterProcessor,
  ) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager'),
      $container->get('file_table_formatter.processor'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'header' => FALSE,
      'use_text_format' => FALSE,
      'text_format' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements['header'] = [
      '#type'           => 'checkbox',
      '#title'          => $this->t('Files include a header row'),
      '#description'    => $this->t('If checked first row will be displayed formatted as a table header'),
      '#default_value'  => $this->getSetting('header'),
    ];

    $elements['use_text_format'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use text format in cells'),
      '#description' => $this->t('Cells display plain text by default. Enabling this lets you select a text format to process cell contents.'),
      '#default_value' => $this->getSetting('use_text_format') ?? FALSE,
    ];

    $text_format_options = [];
    foreach ($this->entityTypeManager->getStorage('filter_format')->loadMultiple() as $format) {
      $text_format_options[$format->id()] = $format->label();
    }

    $elements['text_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Text format'),
      '#description' => $this->t('Process cell contents using this text format.'),
      '#options' => $text_format_options,
      '#default_value' => $this->getSetting('text_format') ?? '',
      '#states' => [
        'visible' => [
          ':input[name$="[settings][use_text_format]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $includes_header = $this->getSetting('header');
    $summary[] = $this->t('Files include a header row: @header', [
      '@header' => empty($includes_header) ? $this->t('No') : $this->t('Yes'),
    ]);
    if (!empty($this->getSetting('use_text_format')) && !empty($this->getSetting('text_format'))) {
      $format = $this->entityTypeManager->getStorage('filter_format')->load($this->getSetting('text_format'));
      $summary[] = $this->t('Use text format: @format', [
        '@format' => $format->label(),
      ]);
    }
    return $summary;
  }
  
  
  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    
    if ($files = $this->getEntitiesToView($items, $langcode)) {
      $includes_header = $this->getSetting('header');
      $text_format = !empty($this->getSetting('use_text_format')) && !empty($this->getSetting('text_format'))
        ? $this->getSetting('text_format')
        : NULL;
      foreach ($files as $delta => $file) {
        $table_data = $this->fileTableFormatterProcessor->getTableDataFromFile($file, $includes_header, $text_format);
        $elements[$delta] = [
          '#theme' => 'file_table_formatter_table',
          '#data' => $table_data,
          '#datatables' => FALSE,
        ];
      }
    }
    return $elements;
  }
  
}
